.module _cpc_PerformClipLine
.globl _cpc_PerformClipLine

;; 0,1 = x1
;; 2,3 = y1
;; 4,5 = x2
;; 6,7 = y2


;; 0 = within region
;; 1 = off min
;; 2 = off max

;; bit 7,6,5,4 = outcode 1 (y1,x1)
;; bit 3,2,1,0 = outcode 2 (y2,x2)

;; regions:
;; 0101 	0100 	0110
;; 0001 	0000 	0010
;; 1001 	1000 	1010

.area _CODE

;;------------------------------------------------------------------------

_cpc_PerformClipLine:
push ix
push iy
ld ix,#6
add ix,sp


ld l,6(ix)			;; y2
ld h,7(ix)
ld e,2(ix)			;; y1
ld d,3(ix)
or a
sbc hl,de
ld (dyc),hl

ld l,4(ix)			;; x2
ld h,5(ix)
ld e,0(ix)			;; x1
ld d,1(ix)
or a
sbc hl,de			;; dx
ld (dxc),hl


;; x1,y1
ld iy,#clipped
ld a,8(ix)
rrca
rrca
rrca
rrca
call PerformClipXY

inc ix
inc ix
inc ix
inc ix

;; x2,y2
ld a,4(ix)
ld iy,#clipped+4
call PerformClipXY

ld hl,#clipped
pop iy
pop ix
ret

;;------------------------------------------------------------------------

PerformClipXY:
;; 0000 = accept

;; 0101 	0100 	0110
;; 0001 	0000 	0010
;; 1001 	1000 	1010

and #0x0f
or a
jp z,pcl1		;; 0
dec a
jp z,pcl2		;; 1
dec a
jp z,pcl3		;; 2
dec a
dec a			;; 4
jp z,pcl4
dec a
jp z,pcl5		;; 5
dec a
jp z,pcl6		;; 6
dec a
dec a
jp z,pcl8		;; 8
dec a
jp z,pcl9		;; 9

pcl10:
ld hl,(region_bottom)
call clip_y
ld 0(iy),l
ld 1(iy),h

;; clip off right side
ld hl,(region_right)
call clip_x
ld 2(iy),l
ld 3(iy),h
jp pcl1done

;;----------------------------------------------------------------------------


pcl9:
ld hl,(region_bottom)
call clip_y
ld 0(iy),l
ld 1(iy),h

;; clip off right side
ld hl,(region_left)
call clip_x
ld 2(iy),l
ld 3(iy),h
jp pcl1done

;;----------------------------------------------------------------------------


pcl8:
ld hl,(region_bottom)
call clip_y
ld 0(iy),l
ld 1(iy),h

ld hl,(region_bottom)
ld 2(iy),l
ld 3(iy),h

jp pcl1done



;;----------------------------------------------------------------------------


pcl6:
ld hl,(region_top)
call clip_y
ld 0(iy),l
ld 1(iy),h

;; clip off right side
ld hl,(region_right)
call clip_x
ld 2(iy),l
ld 3(iy),h
jp pcl1done


;;----------------------------------------------------------------------------


pcl5:
ld hl,(region_top)
call clip_y
ld 0(iy),l
ld 1(iy),h

;; clip off right side
ld hl,(region_left)
call clip_x
ld 2(iy),l
ld 3(iy),h
jp pcl1done

;;----------------------------------------------------------------------------

pcl4:
ld hl,(region_top)
ld 2(iy),l
ld 3(iy),h

ld hl,(region_top)
call clip_y
ld 0(iy),l
ld 1(iy),h
jp pcl1done

;;----------------------------------------------------------------------------

pcl3:
ld hl,(region_right)
ld 0(iy),l
ld 1(iy),h

;; clip off right side
ld hl,(region_right)
call clip_x
ld 2(iy),l
ld 3(iy),h

jp pcl1done
;;----------------------------------------------------------------------------

pcl2:
ld hl,(region_left)
ld 0(iy),l
ld 1(iy),h

;; clip y coord 

;; clip off left side
ld hl,(region_left)
call clip_x
ld 2(iy),l
ld 3(iy),h

jr pcl1done

;;----------------------------------------------------------------------------


pcl1:
;; no clip
ld l,0(ix)
ld h,1(ix)
ld 0(iy),l
ld 1(iy),h

ld l,2(ix)
ld h,3(ix)
ld 2(iy),l
ld 3(iy),h

;;----------------------------------------------------------------------------

pcl1done:
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; x clip
;; f = (xedge - x1)/(x2-x1)
;; y = (y2-y1)*f + y1
;;
;; y = (((xedge-x1)*(y2-y1))/(x2-x1)) + y1

;; HL = edge
clip_x:
ld e,0(ix)			;; x
ld d,1(ix)
or a
sbc hl,de			;; xedge-x
ld (edgesub),hl

;; (y2-y1)*(xedge-x1)
ld bc,(dyc)
ld de,(edgesub)
call Mult16
ld c,l
ld b,h
ld de,(dxc)
call Div16Signed
ld l,c
ld h,b
ld e,2(ix)
ld d,3(ix)
add hl,de
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; y clip
;; f = (yedge - y1)/(y2-y1)
;; x = (x2-x1)*f + x1
;;
;; x = (((yedge-y1)*(x2-x1))/(y2-y1)) + x1

;; HL = edge
clip_y:
ld e,2(ix)			;; y
ld d,3(ix)
or a
sbc hl,de			;; yedge-y
ld (edgesub),hl

;; (x2-x1)*(yedge-y1)
ld bc,(dxc)
ld de,(edgesub)
call Mult16
ld c,l
ld b,h
ld de,(dyc)
call Div16Signed
ld l,c
ld h,b
;; result in hl
ld e,0(ix)
ld d,1(ix)
add hl,de
ret


Mult16:                           ; This routine performs the operation DEHL=BC*DE
  ld hl,#0
  ld a,#16
Mul16Loop:
  add hl,hl
  rl e
  rl d
jp nc,NoMul16
  add hl,bc
  jp nc,NoMul16
  inc de                         ; This instruction (with the jump) is like an "ADC DE,0"
NoMul16:
  dec a
  jp nz,Mul16Loop
  ret

  
Div16Signed:
ld a,b
xor d
and #0x80
;; A = 0 if both same sign, else
;; A !=0 if different sign
push af

bit 7,b
jr z,div161

ld a,c
cpl
ld c,a
ld a,b
cpl
ld b,a
inc bc
div161:
bit 7,d
jr z,div162

ld a,e
cpl
ld e,a
ld a,d
cpl
ld d,a
inc de
div162:

call Div16
pop af
;; - div - -> +ve
;; + div + -> +ve
;; - div + -> -ve
;; + div - -> -ve

ret z

;; neg result
ld a,c
cpl
ld c,a
ld a,b
cpl
ld b,a
inc bc
ret
  
  
;
;Divide 16-bit values (with 16-bit result)
;In: Divide BC by divider DE
;Out: BC = result, HL = rest
;
Div16:	ld hl,#0
	ld a,b
	ld b,#8
Div16_Loop1:
	rla
	adc hl,hl
	sbc hl,de
	jr nc,Div16_NoAdd1
	add hl,de
Div16_NoAdd1:
	djnz Div16_Loop1
	ld b,a
	ld a,c
	ld c,b
	ld b,#8
Div16_Loop2:
	rla
	adc hl,hl
	sbc hl,de
	jr nc,Div16_NoAdd2
	add hl,de
Div16_NoAdd2:
	djnz Div16_Loop2
	rla
	cpl
	ld b,a
	ld a,c
	ld c,b
	rla
	cpl
	ld b,a
	ret	
	
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.area _DATA

dyc:
.dw 0
dxc:
.dw 0
edgesub:
.dw 0


clipped:
.ds 2*4

